@@ -33,7 +33,7 @@ adjust_left_type <- function(x, value)
     S4Vectors:::recycleVector(value, index_len)
 }
 
-.subassign_SVT_by_Lindex <- function(x, Lindex, value)
+.subassign_SVT_by_Lindex <- function(x, Lindex, value, old=FALSE)
 {
     x <- adjust_left_type(x, value)
     stopifnot(is.vector(Lindex), is.numeric(Lindex))
@@ -43,13 +43,19 @@ adjust_left_type <- function(x, value)
         return(x)
 
     value <- .normalize_right_value(value, type(x), length(Lindex))
-    new_SVT <- SparseArray.Call("C_subassign_SVT_by_Lindex",
-                                x@dim, x@type, x@SVT, FALSE, Lindex, value)
+
+    if (old) {
+        new_SVT <- SparseArray.Call("C_subassign_SVT_by_Lindex_OLD",
+                                    x@dim, x@type, x@SVT, Lindex, value)
+    } else {
+        new_SVT <- SparseArray.Call("C_subassign_SVT_by_Lindex",
+                                    x@dim, x@type, x@SVT, FALSE, Lindex, value)
+    }
     BiocGenerics:::replaceSlots(x, SVT=new_SVT, check=FALSE)
 }
 
 setMethod("subassign_Array_by_Lindex", "SVT_SparseArray",
-    .subassign_SVT_by_Lindex
+    function(x, Lindex, value) .subassign_SVT_by_Lindex(x, Lindex, value)
 )
 
 .subassign_SVT_by_Mindex <- function(x, Mindex, value)
@@ -62,8 +68,11 @@ setMethod("subassign_Array_by_Lindex", "SVT_SparseArray",
         return(x)
 
     value <- .normalize_right_value(value, type(x), nrow(Mindex))
-    new_SVT <- SparseArray.Call("C_subassign_SVT_by_Mindex",
-                                x@dim, x@type, x@SVT, FALSE, Mindex, value)
+
+    if (storage.mode(Mindex) != "integer")
+        storage.mode(Mindex) <- "integer"
+    new_SVT <- SparseArray.Call("C_subassign_SVT_by_Mindex_OLD",
+                                x@dim, x@type, x@SVT, Mindex, value)
     BiocGenerics:::replaceSlots(x, SVT=new_SVT, check=FALSE)
 }
 
@@ -81,15 +90,6 @@ setMethod("subassign_Array_by_Mindex", "SVT_SparseArray",
 ### either a NULL or an integer vector of valid indices along the
 ### corresponding dimension in 'x'.
 
-.Nindex2Noffs <- function(Nindex)
-{
-    stopifnot(is.list(Nindex))
-    lapply(Nindex,
-        function(subscript)
-            if (is.null(subscript)) NULL else subscript - 1L
-    )
-}
-
 ### 'Rvector' is considered "short" if it can be cleanly recycled along the
 ### first (a.k.a. leftmost or innermost) dimension of the array selection.
 ### This is a requirement of .subassign_SVT_with_short_Rvector().
@@ -99,9 +99,10 @@ setMethod("subassign_Array_by_Mindex", "SVT_SparseArray",
 ### array selection (in other words it cannot contain zeros).
 ### Returns TRUE or FALSE indicating whether 'Rvector' is considered "short"
 ### or not.
-.is_short <- function(Rvector_len, selection_dim)
+.is_short <- function(Rvector, selection_dim)
 {
-    stopifnot(isSingleInteger(Rvector_len), is.integer(selection_dim))
+    stopifnot(is.vector(Rvector), is.integer(selection_dim))
+    Rvector_len <- length(Rvector)
     if (Rvector_len == 0L)
         stop(wmsg("right value has length zero"))
     selection_dim[[1L]] %% Rvector_len == 0L
@@ -132,35 +133,21 @@ setMethod("subassign_Array_by_Mindex", "SVT_SparseArray",
     if (any(selection_dim == 0L))
         return(x)
 
-    Rvector_len <- length(Rvector)
-    stopifnot(.is_short(Rvector_len, selection_dim))
-
-    ## Prepare 'Noffs' and 'Rvector'.
-    Norder <- S4Arrays:::get_Nindex_order(Nindex)
-    Nindex <- S4Arrays:::subset_Nindex_by_Nindex(Nindex, Norder)
-    Noffs <- .Nindex2Noffs(Nindex)
-    Norder1 <- Norder[[1L]]
-    if (!is.null(Norder1))
-        Rvector <- Rvector[((Norder1 - 1L) %% Rvector_len) + 1L]
-    storage.mode(Rvector) <- new_type
+    stopifnot(.is_short(Rvector, selection_dim))
 
+    storage.mode(Rvector) <- type(x)
     new_SVT <- SparseArray.Call("C_subassign_SVT_with_short_Rvector",
-                                x@dim, x@type, x@SVT, Noffs, Rvector)
+                                x@dim, x@type, x@SVT, Nindex, Rvector)
     BiocGenerics:::replaceSlots(x, SVT=new_SVT, check=FALSE)
 }
 
-.adjust_right_array_dim <- function(right_array, selection_dim)
+.Nindex2Noffs <- function(Nindex)
 {
-    right_dim <- unname(dim(right_array))
-    if (identical(selection_dim, right_dim))
-        return(right_array)
-    effdim_idx1 <- which(selection_dim != 1L)
-    effdim_idx2 <- which(right_dim != 1L)
-    if (!identical(selection_dim[effdim_idx1], right_dim[effdim_idx2]))
-        stop(wmsg("dimensions of right array don't ",
-                  "match dimensions of array selection"))
-    dim(right_array) <- selection_dim
-    right_array
+    stopifnot(is.list(Nindex))
+    lapply(Nindex,
+        function(subscript)
+            if (is.null(subscript)) NULL else subscript - 1L
+    )
 }
 
 .subassign_SVT_with_Rarray <- function(x, Nindex, Rarray)
@@ -175,7 +162,9 @@ setMethod("subassign_Array_by_Mindex", "SVT_SparseArray",
 
     ## No-op (except for type change above) if array selection is empty.
     selection_dim <- S4Arrays:::get_Nindex_lengths(Nindex, x@dim)
-    Rarray <- .adjust_right_array_dim(Rarray, selection_dim)
+    if (!identical(selection_dim, unname(dim(Rarray))))
+        stop(wmsg("dimensions of right array don't ",
+                  "match dimensions of array selection"))
     if (any(selection_dim == 0L))
         return(x)
 
@@ -191,33 +180,35 @@ setMethod("subassign_Array_by_Mindex", "SVT_SparseArray",
     BiocGenerics:::replaceSlots(x, SVT=new_SVT, check=FALSE)
 }
 
-.subassign_SVT_with_SVT <- function(x, Nindex, y)
+.subassign_SVT_with_SVT <- function(x, Nindex, v)
 {
     stopifnot(is(x, "SVT_SparseArray"), is.list(Nindex))
     check_svt_version(x)
-    stopifnot(is(y, "SVT_SparseArray"))
-    check_svt_version(y)
+    stopifnot(is(v, "SVT_SparseArray"))
+    check_svt_version(v)
 
     ## Change 'x' type if necessary.
-    new_type <- type(c(vector(type(x)), vector(type(y))))
+    new_type <- type(c(vector(type(x)), vector(type(v))))
     type(x) <- new_type
 
     ## No-op (except for type change above) if array selection is empty.
     selection_dim <- S4Arrays:::get_Nindex_lengths(Nindex, x@dim)
-    y <- .adjust_right_array_dim(y, selection_dim)
+    if (!identical(selection_dim, unname(dim(v))))
+        stop(wmsg("dimensions of right array don't ",
+                  "match dimensions of array selection"))
     if (any(selection_dim == 0L))
         return(x)
 
-    ## Prepare 'Noffs' and 'y'.
+    ## Prepare 'Noffs' and 'v'.
     Norder <- S4Arrays:::get_Nindex_order(Nindex)
     Nindex <- S4Arrays:::subset_Nindex_by_Nindex(Nindex, Norder)
     Noffs <- .Nindex2Noffs(Nindex)
-    y <- S4Arrays:::subset_by_Nindex(y, Norder)
-    type(y) <- new_type
+    v <- S4Arrays:::subset_by_Nindex(v, Norder)
+    type(v) <- new_type
 
     new_SVT <- SparseArray.Call("C_subassign_SVT_with_SVT",
-                                x@dim, x@type, x@SVT, FALSE, Noffs,
-                                y@dim, y@type, y@SVT, FALSE)
+                                x@dim, x@type, x@SVT, Noffs,
+                                v@dim, v@type, v@SVT)
     BiocGenerics:::replaceSlots(x, SVT=new_SVT, check=FALSE)
 }
 
@@ -256,7 +247,7 @@ setMethod("subassign_Array_by_Mindex", "SVT_SparseArray",
         if (any(selection_dim == 0L))
             return(x)
 
-        if (.is_short(length(value), selection_dim))
+        if (.is_short(value, selection_dim))
             return(.subassign_SVT_with_short_Rvector(x, Nindex, value))
 
         ## Turn 'value' into an ordinary array of same dimensions as
